module Custom.Scratch where
--{- imports
--{- globals
import Custom.Vars
--}
--{- standards
import XMonad
import XMonad.Util.NamedScratchpad
import qualified XMonad.StackSet as W
--}
--}
--{- keybinds and scratches
mscratch' =
 [ "t", "<Return>", "n", "m", "f", "p" ]

mscratch =
 [ NS (s!1) t tF dC
 , NS (s!2) sp spF dC
 , NS (s!3) n nF dC
 , NS (s!4) m mF dC
 , NS (s!5) f fF dC
 , NS (s!6) p pF dC
 ]
 where
  s = mscratch'
  sn = [ "S:" <> (caps x) | x <- s ]
  -- see floatMolds in keys.hs for more info
  dC = customFloating $ mfloatMolds'!1
  t  = mconsole (sn!1) mshell
  tF = resource =? (sn!1) <||> title =? (sn!1)

  sp  = mconsole "S:SP" $ "screen -xaAR scratch -s " <> mshell
  spF = resource =? "S:SP" <||> title =? "S:SP"

  n  = mconsole (sn!3) meditor'  -- assumes CLI
  nF = resource =? (sn!3) <||> title =? (sn!3)

  m  = "vlc"
  mF = resource ~? "vlc"

  f  = mfiles
  fF = className ~? mfiles

  p  = "bitwarden"
  pF = className ~? "bitwarden"
--}
--{- floating boxes
mfloatMolds =
 -- keys designed to work with both qwerty and colemak
 let keys = ["f"] : mwinKeys <> [["S-f"], [",","."],["/"], ["l","i"],[";", "o", "'"]] in

 -- some annoyingly complex mappings to make directional keys work too, add lines in a similar fashion to "f" in order to add more mappings
 -- not all of them need to be mapped, they can be used internally too.
 concat [
 [ (k, mfloatMolds'!i) | k <- keys!i ]
 |  i <- [1 .. length keys ] ]

mfloatMolds' =
 [ bl 0.05 0.05 1    1    -- full  mold
 , bl 0.05 0.05 0.5  1    -- left  mold
 , br 0.05 0.05 0.5  1    -- right mold
 ,  b 0    0    1    1    -- fullscreen float
 , br 0.05 0.05 0.5  0.5  -- popup player float
 , br 0.05 0.05 0.25 0.25 -- popup player float #2
 , bl 0.05 0.05 0.5  0.5  -- popup player float #3
 , bl 0.05 0.05 0.25 0.25 -- popup player float #4
 ]
 where
 b = W.RationalRect
 -- make x and y equal to padding, where the window always left and top aligns
 bl = \x y w h -> b x y (min (w-x) $ 1-x-x) (min (h-y) $ 1-y-y)
 tl = \x y w h -> b x y (min (w-x) $ 1-x-x) (min (h-y) $ 1-y-y)
 -- same as l, but with bottom right anchored x & y
 br = \x y w h -> b (max x $ 1-w) (max y $ 1-h) (max 0.01 (min (w-x) $ 1-x-x)) $ max 0.01 (min (h-y) $ 1-y-y)
 tr = \x y w h -> b (max x $ 1-w) (max y $ 1-h) (max 0.01 (min (w-x) $ 1-x-x)) $ max 0.01 (min (h-y) $ 1-y-y)
--}

{- references
{- -- this tricks vim, enabling syntax highlighting
-}
--}

