{-# LANGUAGE LambdaCase #-}
{-# OPTIONS_GHC -Wno-missing-signatures -Wno-orphans #-}
{- LANGUAGE ViewPatterns #-} -- ?
{- LANGUAGE ParallelListComp #-} -- ?
{- LANGUAGE NoMonomorphismRestriction #-} -- ?
{- LANGUAGE NamedFieldPuns #-} -- ?
{- LANGUAGE FlexibleContexts #-} -- ?

import XMonad.Hooks.FadeWindows (isFloating)
import Control.Monad
import Data.Monoid
-- my xmonad config, see ../.vimrc for navigation
--{- imports
--{- global
import Custom.Grid
import Custom.Keys
import Custom.Prompts
import Custom.Scratch
import Custom.Vars
--}
--{- standards
import XMonad
import qualified XMonad.StackSet as W
import System.IO (hPutStrLn, Handle)
import Control.Monad
--}
--{- data
import Data.Monoid
import Data.List (isInfixOf)
import Data.Char (isSpace, toLower)
import qualified Data.Map as M
import Data.Maybe (fromJust)
--}
--{- actions
import XMonad.Actions.SpawnOn
--}
--{- hooks
import XMonad.Hooks.DynamicLog (dynamicLogWithPP, pad, xmobarPP, xmobarColor, PP(..), shorten)
-- for some fullscreen events, also for xcomposite in obs.
import XMonad.Hooks.EwmhDesktops
import XMonad.Hooks.FadeInactive
import XMonad.Hooks.ManageDocks -- (avoidStruts, docksEventHook, checkDock, manageDocks, ToggleStruts(..))
import XMonad.Hooks.ManageHelpers
import XMonad.Hooks.SetWMName
import XMonad.Hooks.WorkspaceHistory
import XMonad.Hooks.RefocusLast
import XMonad.Hooks.Place
--}
--{- layouts
--{- variants
import qualified XMonad.Layout.Spiral as LS
import XMonad.Layout.GridVariants (Grid(Grid))
import XMonad.Layout.ResizableTile
import XMonad.Layout.Tabbed
import XMonad.Layout.Accordion
import XMonad.Layout.SubLayouts
--}
--{- effects
--{- testing
import XMonad.Layout.Magnifier -- of use
import XMonad.Layout.CenteredMaster -- of use
import qualified XMonad.Layout.HintedTile as HT
--}
import XMonad.Layout.TrackFloating
import XMonad.Layout.LayoutModifier
import XMonad.Layout.LimitWindows (limitWindows)
import XMonad.Layout.NoBorders (smartBorders, noBorders)
import XMonad.Layout.Spacing
import XMonad.Layout.ShowWName
--}
--{- modifiers
import XMonad.Layout.MultiToggle (mkToggle, single, EOT(EOT), (??))
import XMonad.Layout.MultiToggle.Instances (StdTransformers(NBFULL, MIRROR, NOBORDERS))
import XMonad.Layout.Renamed (renamed, Rename(..))
import XMonad.Layout.WindowArranger (windowArrange, WindowArrangerMsg(..))
import qualified XMonad.Layout.ToggleLayouts as T (toggleLayouts, ToggleLayout(Toggle))
import XMonad.Layout.Minimize
import XMonad.Layout.BoringWindows (boringWindows, boringAuto)
--}
--{- actions
import XMonad.Actions.MouseResize
import XMonad.Actions.DynamicProjects
import XMonad.Actions.CopyWindow
import XMonad.Layout.WindowNavigation
--}
--}
--{- utils
import XMonad.Util.EZConfig (additionalKeysP, removeKeysP, checkKeymap)
import XMonad.Util.NamedScratchpad
import XMonad.Util.Run (runProcessWithInput, safeSpawn, spawnPipe)
import XMonad.Util.SpawnOnce
import XMonad.Util.Cursor
--}
--{- urgencyhook
import XMonad.Hooks.UrgencyHook
import XMonad.Util.NamedWindows
import XMonad.Util.Run
--}
--}
--{- autostart
--{- type definitions
mstartupHook :: X ()
--}
mstartupHook = do
 setDefaultCursor xC_left_ptr
 setWMName mwm
 -- note: (spawnpipe mbar) is used in main()
 mapM_ (\x -> spawnOnce $ x ++ " &") $ mlock ++ msession
 mapM_ (\x -> unspawn x <+> spawn ("sleep 1;" ++ x)) $ mtray : mrestart
 where
  mlock =
   [ "xset s " ++ show (60 * mtimeout)
   , "xss-lock " ++ mlocker
   ]
--}
--{- workspaces
--{- type definitions
tagWS :: String -> [(String, (String, X ()))] -> [(String, (String, X ()))]
chop  ::[(String, (String, X ()))] -> [String]
mWS   :: String -> String -> X () -> (String, (String, X ()))
mregular', mfloats', mfulls' :: [(String, (String, X ()))]
mregular , mfloats , mfulls , mworkspaces :: [String]
mprojects :: [Project]
mWSIndices :: M.Map String Int
--}
--{- fn
tagWS tag l = [(a ++ tag, b) | (a, b) <- l ]
chop l = [a | (a, b) <- l]
mWS name directory startup = ( name, (directory, startup) )
mWSIndices = M.fromList $ zipWith (,) mworkspaces [1..]
clickable ws = "<action=xdotool key super+"++show i++">"++ws++"</action>"
 where i = fromJust $ M.lookup ws mWSIndices
--}

--{- regular
mregular' =
 tagWS mreg
 [ mWS "work" "~/"
   $ mapM_ spawnHere [mterminal, mfiles]
 , mWS "dev" "~/"
   $ mapM_ spawnHere [mterminal, meditor ""]
 , mWS "web" "~/"
   $ mapM_ spawnHere [mbrowser]
 , mWS "art" "~/"
   $ mapM_ spawnHere ["gravit-designer", "gimp"]
 , mWS "chat" "~/"
   $ mapM_ spawnHere ["thunderbird", "element-desktop"]
 ]
--}
--{- floats
mfloats' =
 tagWS mfloat
 [ mWS "A1" "~/"
   $ mapM_ spawnHere []
 , mWS "A2" "~/"
   $ mapM_ spawnHere []
 , mWS "A3" "~/"
   $ mapM_ spawnHere []
 ]
--}
--{- fulls
mfulls' =
 tagWS mfull
 [ mWS "B1" "~/"
   $ mapM_ spawnHere ["lutris"]
 , mWS "B2" "~/"
   $ mapM_ spawnHere []
 , mWS "B3" "~/"
   $ mapM_ spawnHere []
 ]
--}

--{- lists
mregular = chop mregular'
mfloats = chop mfloats'
mfulls = chop mfulls'
mworkspaces =
 concat
 [ mregular, mfloats, mfulls
 , ["NSP"]
 ]

mprojects =
 [
 Project
  { projectName = a
  , projectDirectory = b
  , projectStartHook = Just $ do c
  } | (a, (b, c)) <- concat [ mregular', mfloats', mfulls' ]
 ]
--}
--}
--{- hooks
--{- type definitions
mmanageHook :: XMonad.Query (Data.Monoid.Endo WindowSet)
mhandleEventHook :: Event -> X All
mlogHook :: Handle -> X ()
mbarHook :: Handle -> X ()
--mlayoutHook :: String -> String -> l0 Window
--mlayoutHook :: ()
--}
--{- urgencyHook
data LibNotifyUrgencyHook = LibNotifyUrgencyHook deriving (Read, Show)

instance UrgencyHook LibNotifyUrgencyHook where
 urgencyHook LibNotifyUrgencyHook w = do
  name    <- getName w
  Just ws <- fmap (W.findTag w) $ gets windowset
  safeSpawn mnotify' $ mnotify'' <> [ws, show name]

--}
--{- manageHook
{- documentation
tail "abc" -> "bc"
 "Abc123"  ~? "abc" == True   -- tailed infix
 "ABc"     ~? "Abc" == False
 "123abc" ~~? "abc" == True   -- infix
 "abc"    ~~? "Abc" == False
 "abc"     =? "abc" == True   -- exact
 "abc1"    =? "abc" == False

 | operator | false-positivity | verbosity |
 -------------------------------------------
 | ~?       | high             | low       |
 | ~~?      | medium           | medium    |
 | =?       | low              | high      |

usage:
act :: X () -> Functor -> [managehook-thing] -> [[Char]] -> [[Char]] -> ManageHook
act action joining_operator [identifier] [tailed] [exact], eg:
 act (shift 1) (>>|) [className] ["abc"] ["123"] = (never, className ~? "abc" <||> className ~~? "123") --> shift 1
 act (shift 1) (>>&) [className, title] ["urmom"] [] = (never, className ~? "urmom" <&&> title ~? "title") --> shift 1
 act (shift 1) (>>|) [className, title] [] [] = (never) --> shift 1
--}
mmanageHook =
 namedScratchpadManageHook mscratch
 --{- composeOne
 <+> composeOne
  [ doCenterFloat >?| [ isDialog, currentWs ~~? mfloat ]
  , act doCenterFloat (>?|) [className, title]
    [ "picture"
    , "connection"
    , "volume", "pulseaudio", "alsa", "jack"
    , "firefox", "librewolf", "tor Browser", "brave", "chrom", "qute"
    ] []
  , doFullFloat >?| [ isFullscreen, currentWs ~~? mfull ]
  , act doFullFloat (>?|) [className, title] ["steam"] []
  ] --}
 <+> transience'
 --{- composeAll
 <+> composeAll
  -- don't overtake the master window, helps keep focus
  [ doF avoidMaster
  -- , checkDock --> doLower
  , act doCenterFloat (>>|) [className, title]
    [ "pop-up", "overlay", "toolkit", "tool"
    , "dialog", "message", "notif", "zenity"
    , "file", "open", "inspect"
    , "flameshot", "xzoom"
    ] ["man"]
  , act (s 1) (>>|) [className] ["obs"] []
  , act (s 2) (>>|) [className] ["geanie"] []
  , act (s 3) (>>|) [className] ["qute", "librewolf", "brave", "chrom", "tor"] []
  , act (s 4) (>>|) [className] ["krita", "gimp", "design"] []
  , act (s 5) (>>|) [className] ["discord", "thunderbird", "messeng" , "convers"] ["irc", "chat"]

  , act (s' 1) (>>|) [className] ["tor-browser"] []
  , act (s' 2) (>>|) [className, title] ["music", "video"] ["mpv"{-, "vlc"-}]
  , act (s' 3) (>>|) [className] ["xephyr", "virt"] []

  , act (s'' 1) (>>|) [className] ["steam", "lutris"] []
--  , act (ignore >?|) [className, title] ["sharing indicator"] []
  ] --}
 -- <+> manageDocks
 <+> manageSpawn
 --{- where
 where
  never = ( stringProperty "_" =? "_" )
  s n = doShift $ mregular!n
  s' n = doShift $ mfloats!n
  s'' n = doShift $ mfulls!n
  act fn compare values n0 n1 =
   fn `compare`
   concat
   [  [never]
   ++ [x ~? y  | y <- n0]
   ++ [x ~~? y | y <- n1]
      | x <- values ]
  or' a = foldr1 (<||>) a
  and' a = foldr1 (<&&>) a
  (>>|) a b = or' b --> a
  (>>&) a b = and' b --> a
  (>?|) a b = or' b -?> a
  (>?&) a b = and' b -?> a
  -- see above, https://github.com/TheMC47/dotfiles/blob/master/.xmonad/src/Config.hs
  avoidMaster = \ws -> flip W.modify' ws $ \c -> case c of
   W.Stack t [] (r : rs) -> W.Stack t [r] rs
   _                     -> c
  -- pulled from latest manageHelpers, it hasn't been pushed to the nix version
  doLower :: ManageHook
  doLower = ask >>= \w -> liftX $ withDisplay $ \dpy -> io (lowerWindow dpy w) >> mempty
--}
--}
--{- handleEventHook
{- fconf
-- unmodded, mostly (the modifications are cosmetic and allow for more
-- modifications, but don't alter the functionality)
-- floatConfReqHook :: MaybeManageHook -> Event -> X All
-- floatConfReqHook mh ConfigureRequestEvent{ev_window = w} = do
--     runQuery ((<$>) join $ (-?>) isFloating $ mh) w >>= \case
--         Nothing -> mempty
--         Just e -> windows (appEndo e) >> pure (All False)
-- fix attempt
floatConfReqHook :: MaybeManageHook -> Event -> X All
floatConfReqHook mh ConfigureRequestEvent{ev_window = w} = do
    runQuery ((<$>) join $ (-?>) isFloating $ maybeToDefinite mh) w >>= \case
        Nothing -> mempty
        Just e -> windows (appEndo $ Endo (\_ -> e)) >> pure (All False)
floatConfReqHook _ _ = mempty


mfcfrhook :: MaybeManageHook
mfcfrhook = composeAll
    [ appName =? "alarm-clock-applet" -?> doFloat -- prevent alarm-clock-applet from moving its floats to remebered location
    , className =? "Steam" -?> doFloat -- prevent Steam from moving its floats to primary screen
    , className =? "Discord" -?> doFloat
    ]
--}


mhandleEventHook =
 refocusLastWhen isFloat
 <+> XMonad.Hooks.EwmhDesktops.fullscreenEventHook
 <+> docksEventHook
 -- <+> floatConfReqHook mfcfrhook
 -- <+> putBelow "xmobar" (=?) "trayer"
 {- where
 where
  putBelow :: String -> (Query String -> a1 -> Query Bool) -> String -> Event -> X All
  putBelow below compare above ConfigureEvent{ev_window = w, ev_above = a} | a == none = do
   -- when trayer is lowered to the bottom of stack, put all xmobars that
   -- are above it below
   whenX (runQuery (className `compare` above) w) $ do
    withDisplay $ \dpy -> do
     rootw <- asks theRoot
     (_, _, ws) <- io $ queryTree dpy rootw
     let aboveWs = dropWhile (w /=) ws
     belowWs <- filterM (runQuery (appName `compare` below)) aboveWs
     mapM_ (io . lowerWindow dpy) belowWs
   mempty
  putBelow _ _ _ _ = mempty
--}

--}
--{- logHook
mlogHook pipe =
 refocusLastLogHook
 <+> fadeInactiveLogHook 1
 <+> workspaceHistoryHook
 <+> mbarHook pipe
--}
--{- barHook
mbarHook pipe = do
 copies <- wsContainingCopies
 let check x | x `elem` copies = fc (fg!3) (bg!2) $ " + "
                   | otherwise = fc (fg!1) (bg!3) $ (pad . clickable) x
 dynamicLogWithPP . namedScratchpadFilterOutWorkspacePP $ xmobarPP
   { ppOutput = \x -> hPutStrLn pipe x

   , ppSep             = " "
   , ppUrgent          = fc(fg!3) (bg!3) . pad . clickable
   , ppCurrent         = {-xmobarBorder-} fc(bg!1) (fg!2) . pad . clickable
   , ppHidden          = check
   , ppHiddenNoWindows = fc(bg!4) (bg!2) . pad . \x -> take 2 x
   , ppTitle           = fc(fg!5) (bg!2) . pad . shorten 65

   , ppExtras = [windowCount]
   , ppOrder  = \(ws:l:t:ex) -> [l, ws] ++ ex ++ [t]

   }
  where fc = xmobarColor
--}
--{- layoutHook
mlayoutHook mBG mFG =
   mods
   -- windowNavigation .
   -- subLayout [0] (tabs mBG mFG)
 $ accordion
   ||| spiral
   ||| tabs mBG mFG
   -- ||| htile
 where
  mods =
     smartBorders .
     -- withUrgencyHookC BorderUrgencyHook { urgencyBorderColor = mFG } urgencyConfig { suppressWhen = Never } .
     showWName' (mshowWNameTheme mBG mFG) .
     avoidStruts .
     boringAuto .
     -- boringWindows .
     trackFloating .
     mouseResize
--}
--}
--{- layout
--{- type definitions
--}
--{- global settings
mspacing i = spacingRaw True (Border (i*2) (i*2) (i*4) (i*4)) True (Border i i i i) True
nospace = spacingRaw False bnull False bnull False where bnull = Border 0 0 0 0
lay name count space layout =
   windowArrange $
   renamed [Replace name]
 $ (if space > 0 then mspacing space else nospace)
 $ limitWindows count
 $ minimize
-- $ magnifier'
 $ layout
--}
--{- selected layouts
htile =       lay "hti" 6  3 $ HT.HintedTile 2 3 4 HT.Center HT.Tall
accordion =    lay "acc" 8  3 $ Accordion
spiral =       lay "spi" 8  4 $ LS.spiral (3/5)
tabs mBG mFG = lay "tab" 50 0
 $ tabbed shrinkText mtabConfig
 where --{-
  mtabConfig =
   def
   { fontName            = "xft:Mononoki Nerd Font:regular:pixelsize=16"
   , activeColor         = mFG
   , activeBorderColor   = mBG
   , activeTextColor     = mBG

   , inactiveColor       = mBG
   , inactiveBorderColor = mBG
   , inactiveTextColor   = mFG
   } --}
--}
--{- showWorkspaceTheme
mshowWNameTheme mBG mFG =
 def
 { swn_font    = "xft:Mononoki Nerd Font:regular:pixelsize=32"
 , swn_fade    = 0.5
 , swn_bgcolor = mBG
 , swn_color   = mFG
 }
--}
--}
--{- xmonad
--{- type definitions
main :: IO ()
--}
--{- definitions
main = do
 pipe <- spawnPipe mbar
 fgc <-{- liftIO <$>-} rand fg'
 bgc <-{- liftIO <$>-} rand bg'
 let mBG = bgc {-bg!2-}; mFG = fgc {-fg!1-} in
  xmonad
  --}
  --{- hooks  {
  $ docks
  $ withUrgencyHook LibNotifyUrgencyHook
  $ withUrgencyHookC BorderUrgencyHook { urgencyBorderColor = mFG } urgencyConfig { suppressWhen = Never }
--  $ dynamicProjects mprojects
  $ ewmh def
   { manageHook = mmanageHook
   , handleEventHook = mhandleEventHook
   , logHook = mlogHook pipe
   , startupHook = mstartupHook
   , layoutHook  = mlayoutHook mBG mFG
   --}
   --{- settings
   , workspaces         = mworkspaces
   , focusFollowsMouse  = False
   , clickJustFocuses   = True
   , modMask     = super
   , terminal    = mterminal
   --}
   --{- colors }
   , borderWidth        = borderW
   , normalBorderColor  = mBG
   , focusedBorderColor = mFG
   --}
   } --}
   --{- apply keys
   `removeKeysP` [ a | (a,b) <- moverrides]
   `additionalKeysP` moverrides
   `removeKeysP` [ a | (a,b) <- keys]
   `additionalKeysP` keys
  where --{-
   keys =
    ("M-h", checkKeymap (ewmh def `additionalKeysP` mkeys) mkeys) : mkeys
  --}
  --}
--{- type definitions
mshowWNameTheme :: [Char] -> [Char] -> SWNConfig
--}

{- references
{- -- this tricks vim, enabling syntax highlighting
-}
--}
